@@ -29,6 +29,7 @@
 #include "meta-input-settings-private.h"
 
 #include "backends/x11/meta-backend-x11.h"
+#include "meta-cursor-tracker-private.h"
 #include "meta-stage.h"
 
 #ifdef HAVE_NATIVE_BACKEND
@@ -136,6 +137,55 @@ on_device_added (ClutterDeviceManager *device_manager,
   create_device_monitor (backend, device_id);
 }
 
+static inline gboolean
+device_is_slave_touchscreen (ClutterInputDevice *device)
+{
+  return (clutter_input_device_get_device_mode (device) != CLUTTER_INPUT_MODE_MASTER &&
+          clutter_input_device_get_device_type (device) == CLUTTER_TOUCHSCREEN_DEVICE);
+}
+
+static inline gboolean
+check_has_pointing_device (ClutterDeviceManager *manager)
+{
+  const GSList *devices;
+
+  devices = clutter_device_manager_peek_devices (manager);
+
+  for (; devices; devices = devices->next)
+    {
+      ClutterInputDevice *device = devices->data;
+
+      if (clutter_input_device_get_device_mode (device) == CLUTTER_INPUT_MODE_MASTER)
+        continue;
+      if (clutter_input_device_get_device_type (device) == CLUTTER_TOUCHSCREEN_DEVICE ||
+          clutter_input_device_get_device_type (device) == CLUTTER_KEYBOARD_DEVICE)
+        continue;
+
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
+static inline gboolean
+check_has_slave_touchscreen (ClutterDeviceManager *manager)
+{
+  const GSList *devices;
+
+  devices = clutter_device_manager_peek_devices (manager);
+
+  for (; devices; devices = devices->next)
+    {
+      ClutterInputDevice *device = devices->data;
+
+      if (clutter_input_device_get_device_mode (device) != CLUTTER_INPUT_MODE_MASTER &&
+          clutter_input_device_get_device_type (device) == CLUTTER_TOUCHSCREEN_DEVICE)
+        return TRUE;
+    }
+
+  return FALSE;
+}
+
 static void
 on_device_removed (ClutterDeviceManager *device_manager,
                    ClutterInputDevice   *device,
@@ -145,6 +195,32 @@ on_device_removed (ClutterDeviceManager *device_manager,
   int device_id = clutter_input_device_get_device_id (device);
 
   destroy_device_monitor (backend, device_id);
+
+  /* If the device the user last interacted goes away, check again pointer
+   * visibility.
+   */
+  if (backend->current_device_id == device_id)
+    {
+      MetaCursorTracker *cursor_tracker = meta_cursor_tracker_get_for_screen (NULL);
+      gboolean has_touchscreen, has_pointing_device;
+      ClutterInputDeviceType device_type;
+
+      device_type = clutter_input_device_get_device_type (device);
+      has_touchscreen = check_has_slave_touchscreen (device_manager);
+
+      if (device_type == CLUTTER_TOUCHSCREEN_DEVICE && has_touchscreen)
+        {
+          /* There's more touchscreens left, keep the pointer hidden */
+          meta_cursor_tracker_set_pointer_visible (cursor_tracker, FALSE);
+        }
+      else if (device_type != CLUTTER_KEYBOARD_DEVICE)
+        {
+          has_pointing_device = check_has_pointing_device (device_manager);
+          meta_cursor_tracker_set_pointer_visible (cursor_tracker,
+                                                   has_pointing_device &&
+                                                   !has_touchscreen);
+        }
+    }
 }
 
 static MetaMonitorManager *
@@ -177,7 +253,9 @@ meta_backend_real_post_init (MetaBackend *backend)
                                                     NULL, (GDestroyNotify) g_object_unref);
 
   {
+    MetaCursorTracker *cursor_tracker;
     ClutterDeviceManager *manager;
+    gboolean has_touchscreen = FALSE;
     GSList *devices, *l;
 
     /* Create the core device monitor. */
@@ -195,8 +273,12 @@ meta_backend_real_post_init (MetaBackend *backend)
       {
         ClutterInputDevice *device = l->data;
         on_device_added (manager, device, backend);
+        has_touchscreen |= device_is_slave_touchscreen (device);
       }
 
+    cursor_tracker = meta_cursor_tracker_get_for_screen (NULL);
+    meta_cursor_tracker_set_pointer_visible (cursor_tracker, !has_touchscreen);
+
     g_slist_free (devices);
   }
 
@@ -268,6 +350,12 @@ meta_backend_class_init (MetaBackendClass *klass)
                 0,
                 NULL, NULL, NULL,
                 G_TYPE_NONE, 1, G_TYPE_UINT);
+  g_signal_new ("last-device-changed",
+                G_TYPE_FROM_CLASS (object_class),
+                G_SIGNAL_RUN_LAST,
+                0,
+                NULL, NULL, NULL,
+                G_TYPE_NONE, 1, G_TYPE_INT);
 }
 
 static void
@@ -388,6 +476,45 @@ meta_backend_get_stage (MetaBackend *backend)
   return priv->stage;
 }
 
+void
+meta_backend_update_last_device (MetaBackend *backend,
+                                 int          device_id)
+{
+  ClutterInputDeviceType device_type;
+  MetaCursorTracker *cursor_tracker;
+  ClutterDeviceManager *manager;
+  ClutterInputDevice *device;
+
+  if (backend->current_device_id == device_id)
+    return;
+
+  manager = clutter_device_manager_get_default ();
+  device = clutter_device_manager_get_device (manager, device_id);
+
+  if (!device ||
+      clutter_input_device_get_device_mode (device) == CLUTTER_INPUT_MODE_MASTER)
+    return;
+
+  device_type = clutter_input_device_get_device_type (device);
+
+  cursor_tracker = meta_cursor_tracker_get_for_screen (NULL);
+  backend->current_device_id = device_id;
+  g_signal_emit_by_name (backend, "last-device-changed", device_id);
+
+  if (device_type == CLUTTER_KEYBOARD_DEVICE)
+    return;
+
+  switch (device_type)
+    {
+    case CLUTTER_TOUCHSCREEN_DEVICE:
+      meta_cursor_tracker_set_pointer_visible (cursor_tracker, FALSE);
+      break;
+    default:
+      meta_cursor_tracker_set_pointer_visible (cursor_tracker, TRUE);
+      break;
+    }
+}
+
 static GType
 get_backend_type (void)
 {
